#include <stdint.h>

#define FAT32_DIR      (-1)

#define ATTR_READ_ONLY (0x01)
#define ATTR_HIDDEN    (0x02)
#define ATTR_SYSTEM    (0x04)
#define ATTR_VOLUME_ID (0x08)
#define ATTR_DIRECTORY (0x10)
#define ATTR_ARCHIVE   (0x20)
#define ATTR_LONG_NAME (ATTR_READ_ONLY | ATTR_HIDDEN | ATTR_SYSTEM | ATTR_VOLUME_ID)

#define IS_EOC(id)     ((id) >= 0x0FFFFFF8)
#define DIR_ENTRY_SIZE (32)
#define MIN(a, b)      ((a) < (b) ? (a) : (b))

struct fat32 {
	FILE *image;

	// data ulozena na disku
	uint16_t BPB_BytsPerSec;
	uint8_t  BPB_SecPerClus; 
	uint16_t BPB_RsvdSecCnt;
	uint8_t  BPB_NumFATs;
	uint32_t BPB_TotSec32;
	uint32_t BPB_FATSz32;
	uint16_t BPB_ExtFlags;
	uint16_t BPB_FSVer;
	uint32_t BPB_RootClus;
	uint16_t BPB_FSInfo;
	uint8_t  BS_BootSig;
	uint32_t BS_VolID;
	char     BS_VolLab[11];
	char     BS_FilSysType[8];

	// data odvozena
	uint32_t ClusterSz;
	uint32_t FirstDataSector;
	uint32_t DataSec;
	uint32_t CountOfClusters;
};

struct fat32_dir_entry {
	// data ulozena na disku
	char     fileName[11];
	uint8_t  attr;
	uint16_t fstClusterHI;
	uint16_t fstClusterLO;
	uint32_t  fileSize;

	// data odvozena
	uint32_t fstClusterId;
};

struct fat32_fd {
	struct fat32 *fs;
	uint32_t fileOffset;
	uint32_t fileSize;
	uint32_t currentCluster;
	uint32_t bufOffset; // aktualni pozice v bufferu 
	uint32_t bufSize;   // pocet platnych bytu v bufferu 
	int      isDirectory;
	uint8_t  buf[];
};

struct fat32_fd *fat32_open(struct fat32 *fs, uint32_t clusterId, int32_t fileSize);
void fat32_close(struct fat32_fd *fd);
size_t fat32_read(struct fat32_fd *fd, size_t size, uint8_t *dest_buf);
int fat32_read_dir(struct fat32_fd *fd, struct fat32_dir_entry *entry);
void fat32_init(struct fat32 *bpb, FILE *image);
