#include <stdio.h>
#include <stdlib.h>
#include <stdint.h>
#include <string.h>

#include "fat32.h"

/** vypise retezec, ktery je zadany jako posloupnost znaku bez \0 */
static void print_fixstr(char *str, unsigned int size)
{
	for (int i = 0; i < size; i++) {
		printf("%c", str[i]);
	}
}

void print_fs_info(struct fat32 *fs)
{
	printf("Bytes per sec.:\t%i\n", fs->BPB_BytsPerSec);
	printf("Sec. per clus.:\t%i\n", fs->BPB_SecPerClus);
	printf("Rsvd. sec. cnt:\t%i\n", fs->BPB_RsvdSecCnt);
	printf("Num. fats:\t%i\n",      fs->BPB_NumFATs);
	printf("Tot. Sec.:\t%i\n",      fs->BPB_TotSec32);
	printf("Fat size:\t%i\n",       fs->BPB_FATSz32);
	printf("Ext flags:\t%x\n",      fs->BPB_ExtFlags);
	printf("Fs. ver.:\t%x\n",       fs->BPB_FSVer);
	printf("Root clus.:\t%i\n",     fs->BPB_RootClus);
	printf("Fs. info.:\t%i\n",      fs->BPB_FSInfo);
	printf("Boot. sig.\t%c\n",      fs->BS_BootSig ? 'Y' : 'N');
	if (fs->BS_BootSig) {
		printf("Vol. Id.:\t%x\n", fs->BS_VolID);
		printf("Vol. Lab.:\t");  print_fixstr(fs->BS_VolLab, 11);    printf("\n");
		printf("FilSysType:\t"); print_fixstr(fs->BS_FilSysType, 8); printf("\n");
	}
	printf("\n");
	printf("Fst. dat.sec.:\t%i\n",  fs->FirstDataSector);
	printf("Dat. sec. cnt.:\t%i\n", fs->DataSec);
	printf("Cluster cnt.:\t%i\n",   fs->CountOfClusters);
}

/** vypise obsah adresare, ktery je dany cislem clusteru */
void print_dir(struct fat32 *fs, uint32_t dir_cluster)
{
	struct fat32_fd *dir = fat32_open(fs, dir_cluster, FAT32_DIR);
	while (1) {
		struct fat32_dir_entry entry;
		int success = fat32_read_dir(dir, &entry);
		if (!success) break;
		
		print_fixstr(entry.fileName, 11); printf(" %i @ %i\n", entry.fileSize, entry.fstClusterId);
	};
	fat32_close(dir);
}

/** precte obsah textoveho souboru a vypise jej na standardni vystup */
void print_file(struct fat32 *fs, uint32_t file_cluster, uint32_t file_size)
{
	unsigned char buf[8192];
	struct fat32_fd *file = fat32_open(fs, file_cluster, file_size);
	size_t read = fat32_read(file, 8192, buf);
	printf("%zi\n", read);
	for (int i = 0; i < read; i++) {
		char c = buf[i];
		if (c == '\n') printf("\n");
		else if (c < 32) printf(".");
		else printf("%c", c);
		if (i && (i % 80) == 0) printf("\n");
	}
	
	fat32_close(file);

}

int main()
{
	FILE *image = fopen("tutorial10.fat32", "r");

	struct fat32 fs;
	fat32_init(&fs, image);

	print_fs_info(&fs);
	print_dir(&fs, fs.BPB_RootClus);
	print_file(&fs, 3, 3862); // 3 -- odpovida prvnimu clusteru souboru

	fclose(image);
}
